/*
fassade blinds v. 1.05 for 3DS MAX 4
by Christian Bauer (christian@cgtechniques.com)

based on blinds 1.0 by Jon Seagull (jon@jonseagull.com):
thanks to Alex Bicalho for help with the create tool

creates blinds for the whole fassade.


How to use:
1. give the plugin the numbers of floors
2. Drag the base along the first floor 
3. drag the height



r5: 
now you can move the blinds also in front of the glasfassade
removed the unused height parameter

r4: initial public release


*/

plugin simpleobject Fassade 
name:"Fassade" 
classID:#(0x651972dc, 0x254a2c42)
version:1
category:"Blinds" 
( 
--declare variables
local startpoint
--------------------------------------
--declare functions
--------------------------------------
--parameter block
parameters main rollout:params
	(
	blindtype type:#integer ui:ui_blindtype default:1 
	--height type:#worldunits ui:ui_height
	width type:#worldunits ui:ui_width
	behind type:#worldunits ui:ui_behind
	--twist type:#float ui:ui_twist default:0	
	slats type:#integer ui:ui_slats default:24
	slatwidth type:#worldunits ui:ui_slatwidth default:0.2
	open_percent type:#percent ui:ui_open_percent default:0
	open_treshold type:#percent ui:ui_open_treshold default:0
	
	floors type:#integer ui:ui_floors default:6
	roomheight type:#worldunits ui:ui_roomheight default:3.2
	ceilingheight type:#worldunits ui:ui_ceilingheight default:0.3
	rnd_seed type:#integer ui:ui_rnd_seed default:1234	
	)
-------------------------------------
--rollouts

rollout params "Settings" width:162 height:309
	(
	dropdownlist ui_blindtype "Type: " items:#("Random-open", "Sunny side", "Clouded")
	
	spinner ui_behind "Behind Glass:" type:#worldunits range:[-99999,99999,0]
	
	spinner ui_width "Width:" type:#worldunits range:[0,99999,width]
	--spinner ui_twist "Twist: " type:#float scale:.1 range:[-89,89,0]
	spinner ui_slats "# of Slats: " type:#integer range:[2,255,24]
	spinner ui_slatwidth "Slat seam: " type:#worldunits range:[0,99999,slatwidth]
	label lbl "Building:"
	-- spinner ui_height "Height: " type:#worldunits range:[0,9999,height]	
	spinner ui_floors "# of floors: " type:#integer range:[2,255,24]
	spinner ui_roomheight "roomheight: " type:#worldunits range:[0,9999,roomheight]	
	spinner ui_ceilingheight "ceilingthickness: " type:#worldunits range:[0,9999,ceilingheight]	
	label lb2 " "

	spinner ui_open_percent "Open %: " type:#float scale:1 range:[0,99,0]
	spinner ui_open_treshold "treshold: " type:#float scale:1 range:[0,99,0]
	spinner ui_rnd_seed "seed : " type:#integer scale:1 range:[0,999999,1234]
	
	label lb3 "Creation tips:"
	label lbl01 "Hold Ctrl / Shift to constrain angle"
	label lbl03 "Hold Alt to set width slower"
	
	on ui_blindtype selected val do
		(
				rnd_seed =(random 0 99999)

				ui_slats.enabled = true
				ui_slatwidth.enabled = true
				ui_slatwidth.enabled = true
				ui_roomheight.enabled = true
				ui_ceilingheight.enabled = true
				ui_floors.enabled = true

		)--end handler
	)--end rollout params
	
rollout info "About" width:162 height:146
(
	label lbl1 "Fassade blinds 1.0 for max/viz" pos:[5,3] 
	label lbl1a "by Christian Bauer" pos:[5,23]
	label lbl1b "christian@cgtechniques.com" pos:[5,38]
	label lbl2 "based on Blinds by Jon Seagull" pos:[5,60]
	label lbl2a "jon@jonseagull.com" pos:[5,78]
	label lbl4 "Dedicated to the victims of 911" pos:[5,110]
	
	label lbl7 "..but: The 'War on Terror' Must" pos:[5,130]
    label lbl8 "Not Be an Excuse to"  pos:[35,145]
	label lbl9 "Deny Human Rights !!" pos:[35,160]


)

-------------------------------------
--buildmesh

on buildmesh do
(
	
	
	seed rnd_seed
	local spacing = width/slats
	
	local pct = open_percent/100
	--print "doit"
	verts=#()
	faces=#()
	local rnd
	local ctr
	for i in 1 to slats do
		(
				



		for e in 1 to floors do
		(
				
				
		case blindtype of
		(
			1: --chris test blinds
			(
		
			rnd = (random open_percent 80)
			if rnd >= (99-open_treshold) then rnd = 99
			ctr = ceilingheight-(roomheight*rnd)/100
			if ctr >= 0 then ctr = 0
			)
				
				
			2: --sunny
			(
			rnd = (random 0 open_percent)
			if rnd >= (99-open_treshold) then rnd = 98
			ctr = ceilingheight-(roomheight*rnd)/100
			if ctr >= 0 then ctr = 0

			)
				
					
			3: --clouded
			(
			rnd = (random 130 (30+open_percent))
			if rnd >= (99-open_treshold) then rnd = 98
			ctr = ceilingheight-(roomheight*rnd)/100
			if ctr >= 0 then ctr = 0
			)
		)--end case
				
		
   		local SlatVerts = #(
		[spacing*(i-1),behind,roomheight*(e-1)-ctr],
		[(spacing*i)-slatwidth,behind,roomheight*(e-1)-ctr],
		[spacing*(i-1),behind,-ceilingheight+(roomheight*e)],
		[(spacing*i)-slatwidth,behind,-ceilingheight+(roomheight*e)],
		[spacing*(i-1),behind+1,roomheight*(e-1)-ctr],
		[(spacing*i)-slatwidth,behind+1,roomheight*(e-1)-ctr],
		[spacing*(i-1),behind+1,-ceilingheight+(roomheight*e)],
		[(spacing*i)-slatwidth,behind+1,-ceilingheight+(roomheight*e)]
		)
		
		local fi = verts.count+1
		for v in 1 to slatverts.count do append verts slatVerts[v]
		local slatfaces = #([fi,fi+1,fi+2],[fi+3,fi+2,fi+1])
		for f in 1 to slatfaces.count do append faces slatfaces[f]
				
				
		)--end for e
	)--end for i


	
	local obj = TriMesh() 
	local tvarray = #([0,0,0],[1,0,0],[0,1,0],[1,1,0])
	SetMesh obj numverts:verts.count numfaces:faces.count 
	SetMesh obj verts:verts 
	SetMesh obj faces:faces 
	local mat =#()
	for i in 1 to faces.count do
		(
		append mat blindtype
		)--end for
	SetMesh obj materialIDs:mat 
	update obj 
	
	SetMesh mesh obj 
	local sgarray=#(1,1,2,2)
	local sgindex=1
	for face = 1 to mesh.numfaces do 
		(
		setfaceSmoothGroup mesh face sgarray[sgindex]
		if sgindex < 4 then sgindex+=1 else sgindex = 1
		for edge = 1 to 3 do 
			(
			if edge==2 then setEdgeVis mesh face edge false 
			else setEdgeVis mesh face edge true 
			)--end for edge
		)--end for face
	mesh.numtverts = mesh.numverts
	buildTVfaces mesh


	tvarray_idx=1
	for v in 1 to mesh.numverts do
		(
		setTVert mesh v tvarray[tvarray_idx]
		if tvarray_idx<4 then tvarray_idx+=1 else tvarray_idx=1
		)--end for v
	for f = 1 to mesh.numfaces do setTVface mesh f (getface mesh f)
	update mesh
	
	
	
	-- height = roomheight*floors
	
	
	)--end buildmesh







tool create 
	(
	
	-- Need to declare ST as local
	
	local ST
	local my_euler = eulerangles 0 0 0
	local gdx
	local gdy


		
	on mousePoint click do case click of 
		(
		
		-- Initialize ST variable to record the initial point clicked
		
		1: 
		(
		nodeTM.translation = st = gridPoint 
		nodeTM.rotation = quat 0 0 1 0
		)
		
		3:#stop
		) -- end MousePoint

	on mouseMove click do case click of 
		( 
		
		2: 	(
			
			-- width -------------------------
	
			width = (abs(griddist.x * cos(gridangle.z))) + (abs(griddist.y * sin(gridangle.z)))
			
			--rotation -------------------------
			my_euler = eulerangles 0 0 gridangle.z
			
			if shiftKey then --constrain to cardinal directions
				(
				local grid_z = gridangle.z
				case of
					(
					(grid_z<=45 and -45<grid_z): my_euler = eulerangles 0 0 0
					(45<grid_z and grid_z<=135): my_euler = eulerangles 0 0 90
					(135<grid_z or grid_z<=-135): my_euler = eulerangles 0 0 180
 					(-135<grid_z and grid_z<=-45): my_euler = eulerangles 0 0 270
					)--end case
				)--end if
				
			if ctrlKey then --constrains to 15 degree increments
				(
				local index, sign, the_angle
				local angle_array = #(0,15,30,45,60,75,90,105,120,135,150,165,180)
				if gridangle.z >= 0 then sign=1 else sign=2
				index = floor(((abs(gridangle.z)+15)/15)+.5)
				the_angle = angle_array[index]
				case sign of 
					(
					1: my_euler = eulerangles 0 0 the_angle
					2: my_euler = eulerangles 0 0 -the_angle
					)--end case
				)--end if
			
			nodeTM.rotation = my_euler as quat
			
			-- fix position ------------------
			nodeTM.translation = st		
			)


		
		3:  (
			height = abs(distance gridDist [0,0,0])
			roomheight = height/floors
			-- floors = (distance gridDist [0,0,0]) / roomheight
			
			)
	

		
		) -- end MouseMove

	) -- end tool create






















	
) -- end

